@@ -5,7 +5,9 @@ from functools import lru_cache
 from typing import Dict, Iterable, List, Optional, Union
 
 import numpy as np
+import ray
 from pymatgen.core.composition import Composition, Element
+from tqdm import tqdm
 
 from rxn_network.core.calculator import Calculator
 from rxn_network.core.cost_function import CostFunction
@@ -17,6 +19,17 @@ from rxn_network.enumerators.minimize import (
 )
 from rxn_network.reactions.computed import ComputedReaction
 from rxn_network.reactions.reaction_set import ReactionSet
+from rxn_network.utils import initialize_ray, to_iterator
+
+
+@ray.remote
+def _calculate_ray(obj, rxn):
+    return obj.calculate(rxn)
+
+
+@ray.remote
+def _decorate_ray(obj, rxn):
+    return obj.decorate(rxn)
 
 
 class CompetitivenessScoreCalculator(Calculator):
@@ -40,6 +53,7 @@ class CompetitivenessScoreCalculator(Calculator):
         use_minimize=False,
         basic_enumerator_kwargs: Optional[Dict] = None,
         minimize_enumerator_kwargs: Optional[Dict] = None,
+        quiet=True,
         name: str = "c_score",
     ):
         """
@@ -70,6 +84,10 @@ class CompetitivenessScoreCalculator(Calculator):
         if not self.minimize_enumerator_kwargs.get("calculators"):
             self.minimize_enumerator_kwargs["calculators"] = calcs
 
+        if quiet:
+            self.basic_enumerator_kwargs["quiet"] = True
+            self.minimize_enumerator_kwargs["quiet"] = True
+
     def calculate(self, rxn: ComputedReaction) -> float:
         """
         Calculates the competitiveness score for a given reaction by enumerating
@@ -92,6 +110,53 @@ class CompetitivenessScoreCalculator(Calculator):
 
         return c_score
 
+    def calculate_many(self, rxns: List[ComputedReaction]) -> List[float]:
+        """
+        Calculates the competitiveness score for a list of reactions by enumerating
+        competing reactions, evaluating their cost with the supplied cost function, and
+        then using the c-score formula, i.e. the _get_c_score() method, to determine the
+        competitiveness score. Parallelized with ray.
+
+        Args:
+            rxns: the list of ComputedReaction objects to be evaluated
+
+        Returns:
+            The list of competitiveness scores
+        """
+        initialize_ray()
+        obj = ray.put(self)
+
+        costs = [_calculate_ray.remote(obj, rxn) for rxn in rxns]
+        iterator = tqdm(to_iterator(costs), total=len(costs))
+
+        results = []
+        for r in iterator:
+            results.append(r)
+
+        return results
+
+    def decorate_many(self, rxns: List[ComputedReaction]) -> List[ComputedReaction]:
+        """
+        Decorates a list of reactions with the competitiveness score. Parallelized with
+        ray.
+
+        Args:
+            rxns: the list of ComputedReaction objects to be decorated
+
+        Returns:
+            The list of decorated ComputedReaction objects
+        """
+        obj = ray.put(self)
+
+        new_rxns = [_decorate_ray.remote(obj, rxn) for rxn in rxns]
+        iterator = tqdm(to_iterator(new_rxns), total=len(new_rxns))
+
+        results = []
+        for r in iterator:
+            results.append(r)
+
+        return results
+
     @lru_cache(maxsize=1)
     def get_competing_rxns(self, rxn: ComputedReaction) -> List[ComputedReaction]:
         """
@@ -105,6 +170,8 @@ class CompetitivenessScoreCalculator(Calculator):
             A list of competing reactions
 
         """
+        chemsys = rxn.chemical_system.split("-")
+
         precursors = [r.reduced_formula for r in rxn.reactants]
         open_elem = self.open_elem
 
@@ -116,6 +183,9 @@ class CompetitivenessScoreCalculator(Calculator):
         if open_phases:
             precursors = list(set(precursors) - set(open_phases))
 
+        entries = self.entries.filter_by_stability(0.0).get_subset_in_chemsys(chemsys)
+        entries.update(rxn.entries)  # add back entries which may have been filtered out
+
         enumerators = []
         if self.use_basic:
             kwargs = self.basic_enumerator_kwargs.copy()
@@ -143,7 +213,7 @@ class CompetitivenessScoreCalculator(Calculator):
 
         rxns = set()
         for e in enumerators:
-            rxns.update(e.enumerate(self.entries))
+            rxns.update(e.enumerate(entries))
 
         if rxn in rxns:
             rxns.remove(rxn)
