@@ -6,6 +6,7 @@ add extra functionality.
 import hashlib
 from itertools import combinations
 from typing import List, Optional
+from functools import cached_property
 
 import numpy as np
 from monty.json import MontyDecoder
@@ -21,6 +22,20 @@ from scipy.interpolate import interp1d
 from rxn_network.data import G_ELEMS
 
 
+# TO-DO: Find better method for caching formula than monkey patching a class from pymatgen.
+@cached_property  # type: ignore
+def cached_reduced_formula(self):
+    """
+    Simple caching for Composition's reduced formula. Due to repetitive calls to this
+    property, this can offer a significant speedup.
+    """
+    return self.get_reduced_formula_and_factor()[0]
+
+
+Composition.reduced_formula = cached_reduced_formula  # type: ignore # noqa
+Composition.reduced_formula.__set_name__(Composition, "reduced_formula")  # type: ignore # noqa
+
+
 class GibbsComputedEntry(ComputedEntry):
     """
     An extension to ComputedEntry which estimates the Gibbs free energy of formation
