@@ -7,9 +7,11 @@ from itertools import combinations, product
 from math import comb
 from typing import List, Optional, Set
 
+import ray
+from numpy import median
 from pymatgen.analysis.phase_diagram import GrandPotentialPhaseDiagram, PhaseDiagram
 from pymatgen.entries.computed_entries import ComputedEntry
-from tqdm.auto import tqdm
+from tqdm import tqdm
 
 from rxn_network.core import Enumerator
 from rxn_network.entries.entry_set import GibbsEntrySet
@@ -20,7 +22,10 @@ from rxn_network.enumerators.utils import (
     initialize_entry,
 )
 from rxn_network.reactions import ComputedReaction
-from rxn_network.utils import limited_powerset
+from rxn_network.utils import initialize_ray, limited_powerset, to_iterator
+
+
+PARALLEL_THRESHOLD = 4  # median computation size above which parallelization enabled
 
 
 class BasicEnumerator(Enumerator):
@@ -42,6 +47,7 @@ class BasicEnumerator(Enumerator):
         exclusive_targets: bool = False,
         remove_unbalanced: bool = True,
         remove_changed: bool = True,
+        calculate_e_above_hulls: bool = True,
         quiet: bool = False,
     ):
         """
@@ -81,11 +87,10 @@ class BasicEnumerator(Enumerator):
         self.exclusive_targets = exclusive_targets
         self.remove_unbalanced = remove_unbalanced
         self.remove_changed = remove_changed
+        self.calculate_e_above_hulls = calculate_e_above_hulls
         self.quiet = quiet
 
         self._stabilize = False
-        if "ChempotDistanceCalculator" in self.calculators:
-            self._stabilize = True  # ChempotDistanceCalculator requires stable entries
 
         self.open_phases: Optional[List] = None
         self._build_pd = False
@@ -97,7 +102,7 @@ class BasicEnumerator(Enumerator):
         was initialized with specified precursors or target, the reactions will be
         filtered by these constraints. Every enumerator follows a standard procedure:
 
-        1. Initialize entries, i.e. ensure that precursors and target are considered
+        1. Initialize entries, i.e., ensure that precursors and target are considered
         stable entries within the entry set. If using ChempotDistanceCalculator,
         ensure that entries are filtered by stability.
 
@@ -116,39 +121,65 @@ class BasicEnumerator(Enumerator):
         Args:
             entries: the set of all entries to enumerate from
         """
+
         entries, precursors, targets, open_entries = self._get_initialized_entries(
             entries
         )
 
         combos_dict = self._get_combos_dict(entries, precursors, targets, open_entries)
         open_combos = self._get_open_combos(open_entries)
+        if not open_combos:
+            open_combos = []
 
-        pbar = tqdm(
-            combos_dict.items(), desc=self.__class__.__name__, disable=self.quiet
-        )
+        items = sorted(
+            combos_dict.items(), key=lambda e: len(e[1]), reverse=True
+        )  # sorted for better parallelization
 
-        rxns = []
-        for chemsys, combos in pbar:
-            pbar.set_description(f"{chemsys}")
+        parallel = False
+        median_comp_size = median([len(v) for v in combos_dict.values()])
 
-            elems = chemsys.split("-")
+        if median_comp_size > PARALLEL_THRESHOLD and len(combos_dict) > 1:
+            parallel = True
 
-            filtered_entries = entries.get_subset_in_chemsys(elems)
-            calculators = initialize_calculators(self.calculators, filtered_entries)
+            initialize_ray()
+            obj = ray.put(self)
+            open_combos = ray.put(open_combos)
+            entries = ray.put(entries)
+            open_entries = ray.put(open_entries)
+            precursors = ray.put(precursors)
+            targets = ray.put(targets)
 
-            rxn_iter = self._get_rxn_iterable(combos, open_combos)
+            rxns = [
+                _get_rxns_from_chemsys_ray.remote(
+                    obj, item, open_combos, entries, open_entries, precursors, targets
+                )
+                for item in items
+            ]
+        else:
+            rxns = []
+            iterator = items
+            if not self.quiet:
+                iterator = tqdm(items, total=len(items))
+            for item in items:
+                rxns.append(
+                    self._get_rxns_in_chemsys(
+                        item, open_combos, entries, open_entries, precursors, targets
+                    )
+                )
 
-            r = self._get_rxns(
-                rxn_iter,
-                precursors,
-                targets,
-                calculators,
-                filtered_entries,
-                open_entries,
-            )
-            rxns.extend(r)
+        results = []
 
-        return list(set(rxns))
+        iterator = rxns
+
+        if parallel:
+            iterator = to_iterator(rxns)
+            if not self.quiet:
+                iterator = tqdm(iterator, total=len(rxns))
+
+        for r in iterator:
+            results.extend(r)
+
+        return list(set(results))
 
     def estimate_max_num_reactions(self, entries: List[ComputedEntry]) -> int:
         """
@@ -195,7 +226,35 @@ class BasicEnumerator(Enumerator):
         _ = (self, open_entries)  # unused_arguments
         return None
 
-    def _get_rxns(
+    def _get_rxns_in_chemsys(
+        self,
+        item,
+        open_combos,
+        entries,
+        open_entries,
+        precursors,
+        targets,
+    ):
+        """ """
+        chemsys, combos = item
+
+        elems = chemsys.split("-")
+        filtered_entries = entries.get_subset_in_chemsys(elems)
+        calculators = initialize_calculators(self.calculators, filtered_entries)
+
+        rxn_iter = self._get_rxn_iterable(combos, open_combos)
+
+        rxns = self._get_rxns_from_iterable(
+            rxn_iter,
+            precursors,
+            targets,
+            calculators,
+            filtered_entries,
+            open_entries,
+        )
+        return rxns
+
+    def _get_rxns_from_iterable(
         self,
         rxn_iterable,
         precursors,
@@ -248,12 +307,14 @@ class BasicEnumerator(Enumerator):
             ):
                 continue
 
-            if not (precursor_func(r) or precursor_func(p)):
+            if not (precursor_func(r) or (p and precursor_func(p))):
                 continue
             if p and not (target_func(r) or target_func(p)):
                 continue
 
-            suggested_rxns = self._react(r, p, calculators, pd, grand_pd)
+            suggested_rxns = self._react(
+                r, p, calculators, filtered_entries, pd, grand_pd
+            )
 
             for rxn in suggested_rxns:
                 if (
@@ -271,12 +332,25 @@ class BasicEnumerator(Enumerator):
 
         return rxns
 
-    def _react(self, reactants, products, calculators, pd=None, grand_pd=None):
+    def _react(
+        self,
+        reactants,
+        products,
+        calculators,
+        filtered_entries=None,
+        pd=None,
+        grand_pd=None,
+    ):
         """
         Generates reactions from a list of reactants, products, and optional
         calculator(s)
         """
-        _ = (pd, grand_pd, self)  # unused arguments in BasicEnumerator class
+        _ = (
+            filtered_entries,
+            pd,
+            grand_pd,
+            self,
+        )  # unused arguments in BasicEnumerator class
 
         forward_rxn = ComputedReaction.balance(reactants, products)
         backward_rxn = forward_rxn.reverse()
@@ -295,16 +369,19 @@ class BasicEnumerator(Enumerator):
 
     def _get_initialized_entries(self, entries):
         """Returns initialized entries, precursors, target, and open entries"""
-        precursors, targets = set(), set()
-        entries_new = deepcopy(entries)
 
         def initialize_entries_list(ents):
             new_ents = {initialize_entry(f, entries, self.stabilize) for f in ents}
             return new_ents
 
+        precursors, targets = set(), set()
+
+        entries_new = GibbsEntrySet(
+            deepcopy(entries), calculate_e_above_hulls=self.calculate_e_above_hulls
+        )
+
         if self.precursors:
             precursors = initialize_entries_list(self.precursors)
-
         if self.targets:
             targets = initialize_entries_list(self.targets)
 
@@ -315,7 +392,7 @@ class BasicEnumerator(Enumerator):
                         e.composition.reduced_formula
                     )
                     entries_new.discard(old_e)
-                except IndexError:
+                except KeyError:
                     pass
 
                 entries_new.add(e)
@@ -414,6 +491,7 @@ class BasicOpenEnumerator(BasicEnumerator):
         exclusive_targets: bool = False,
         remove_unbalanced: bool = True,
         remove_changed: bool = True,
+        calculate_e_above_hulls: bool = False,
         quiet: bool = False,
     ):
         """
@@ -474,7 +552,7 @@ class BasicOpenEnumerator(BasicEnumerator):
             num_combos = comb(len(entries), i)
             num_total_combos += num_combos_with_open * num_combos
 
-        return num_total_combos ** 2
+        return num_total_combos**2
 
     def _get_open_combos(self, open_entries):
         """Get all possible combinations of open entries. For a single entry,
@@ -496,3 +574,16 @@ class BasicOpenEnumerator(BasicEnumerator):
         rxn_iter = product(combos, combos_with_open)
 
         return rxn_iter
+
+
+@ray.remote
+def _get_rxns_from_chemsys_ray(
+    obj, item, open_combos, entries, open_entries, precursors, targets
+):
+    """
+    Wrapper function for parallelizing reaction enumeration using ray. See
+    _get_rxns_in_chemsys in BasicEnumerator for more details.
+    """
+    return obj._get_rxns_in_chemsys(  # pylint: disable=protected-access
+        item, open_combos, entries, open_entries, precursors, targets
+    )
