@@ -5,8 +5,9 @@ import itertools
 import warnings
 from typing import Iterable, List, Optional, Union
 
-from fireworks import FiretaskBase, FWAction, explicit_serialize
+from fireworks import FWAction, FiretaskBase, explicit_serialize
 from maggma.stores import MongoStore
+from monty.serialization import dumpfn
 from pymatgen.core.structure import Structure
 from pymatgen.entries import Entry
 from pymatgen.entries.compatibility import MaterialsProject2020Compatibility
@@ -51,6 +52,7 @@ class EntriesFromMPRester(FiretaskBase):
     optional_params = [
         "include_nist_data",
         "include_barin_data",
+        "include_freed_data",
         "include_polymorphs",
     ]
 
@@ -60,11 +62,12 @@ class EntriesFromMPRester(FiretaskBase):
         e_above_hull = self["e_above_hull"]
         include_nist_data = self.get("include_nist_data", True)
         include_barin_data = self.get("include_barin_data", False)
+        include_freed_data = self.get("include_freed_data", False)
         include_polymorphs = self.get("include_polymorphs", False)
 
         with MPRester() as mpr:
             entries = mpr.get_entries_in_chemsys(
-                elements=chemsys, inc_structure="final"
+                elements=chemsys, inc_structure="final", property_data=["icsd_ids"]
             )
 
         entries = process_entries(
@@ -72,10 +75,13 @@ class EntriesFromMPRester(FiretaskBase):
             temperature=temperature,
             include_nist_data=include_nist_data,
             include_barin_data=include_barin_data,
+            include_freed_data=include_freed_data,
             e_above_hull=e_above_hull,
             include_polymorphs=include_polymorphs,
         )
-        return FWAction(update_spec={"entries": entries})
+
+        dumpfn(entries, "entries.json.gz")
+        return FWAction(update_spec={"entries_fn": "entries.json.gz"})
 
 
 @explicit_serialize
@@ -113,6 +119,7 @@ class EntriesFromDb(FiretaskBase):
     optional_params = [
         "include_nist_data",
         "include_barin_data",
+        "include_freed_data",
         "include_polymorphs",
         "inc_structure",
         "compatible_only",
@@ -125,6 +132,7 @@ class EntriesFromDb(FiretaskBase):
         temperature = self["temperature"]
         include_nist_data = self.get("include_nist_data", True)
         include_barin_data = self.get("include_barin_data", False)
+        include_freed_data = self.get("include_freed_data", False)
         e_above_hull = self["e_above_hull"]
         include_polymorphs = self.get("include_polymorphs", False)
         inc_structure = self.get("inc_structure", "final")
@@ -146,10 +154,12 @@ class EntriesFromDb(FiretaskBase):
             temperature=temperature,
             include_nist_data=include_nist_data,
             include_barin_data=include_barin_data,
+            include_freed_data=include_freed_data,
             e_above_hull=e_above_hull,
             include_polymorphs=include_polymorphs,
         )
-        return FWAction(update_spec={"entries": entries})
+        dumpfn(entries, "entries.json.gz")
+        return FWAction(update_spec={"entries_fn": "entries.json.gz"})
 
 
 def process_entries(
@@ -157,6 +167,7 @@ def process_entries(
     temperature: float,
     include_nist_data: bool,
     include_barin_data: bool,
+    include_freed_data: bool,
     e_above_hull: float,
     include_polymorphs: bool,
 ) -> GibbsEntrySet:
@@ -182,6 +193,7 @@ def process_entries(
         temperature=temperature,
         include_nist_data=include_nist_data,
         include_barin_data=include_barin_data,
+        include_freed_data=include_freed_data,
     )
     entry_set = entry_set.filter_by_stability(
         e_above_hull=e_above_hull, include_polymorphs=include_polymorphs
@@ -441,6 +453,7 @@ def get_entry_task(
     e_above_hull = entry_set_params.get("e_above_hull", 0.0)
     include_nist_data = entry_set_params.get("include_nist_data", True)
     include_barin_data = entry_set_params.get("include_barin_data", False)
+    include_freed_data = entry_set_params.get("include_freed_data", False)
     include_polymorphs = entry_set_params.get("include_polymorphs", False)
 
     if bool(entry_db_file):
@@ -450,8 +463,10 @@ def get_entry_task(
             temperature=temperature,
             include_nist_data=include_nist_data,
             include_barin_data=include_barin_data,
+            include_freed_data=include_freed_data,
             e_above_hull=e_above_hull,
             include_polymorphs=include_polymorphs,
+            property_data=["icsd_ids"],
         )
     else:
         entry_task = EntriesFromMPRester(
@@ -459,6 +474,7 @@ def get_entry_task(
             temperature=temperature,
             include_nist_data=include_nist_data,
             include_barin_data=include_barin_data,
+            include_freed_data=include_freed_data,
             e_above_hull=e_above_hull,
             include_polymorphs=include_polymorphs,
         )
