@@ -1,20 +1,20 @@
-" A balanced reaction pathway class"
-from functools import cached_property
-from typing import List, Optional, Union
+"""
+Implements a class for storing balanced reaction pathways.
+"""
+from typing import List, Union
 
 import numpy as np
+from pymatgen.core.composition import Composition
 
 from rxn_network.core import Pathway, Reaction
 from rxn_network.pathways.basic import BasicPathway
-from rxn_network.pathways.utils import balance_path_arrays
-from rxn_network.reactions.basic import BasicReaction
 from rxn_network.utils import limited_powerset
 
 
 class BalancedPathway(BasicPathway):
     """
     Helper class for storing multiple ComputedReaction objects which form a single
-    reaction pathway as identified via pathfinding methods. Includes cost of each
+    reaction pathway as identified via pathfinding methods. Includes costs for each
     reaction.
     """
 
@@ -22,22 +22,20 @@ class BalancedPathway(BasicPathway):
         self,
         reactions: List[Reaction],
         coefficients: List[float],
-        costs: Optional[List[float]] = None,
-        balanced: Optional[bool] = None,
+        costs: List[float],
+        balanced: bool = False,
     ):
         """
         Args:
-            rxns ([ComputedReaction]): list of ComputedReaction objects in pymatgen
-                which occur along path.
-            costs ([float]): list of corresponding costs for each reaction.
+            reactions: list of ComputedReaction objects which occur along path.
+            coefficients: list of coefficients to balance each of these reactions, respectively
+            costs: list of corresponding costs for each reaction.
+                balanced: whether or not the reaction pathway is balanced. Defaults to False.
         """
         self.coefficients = coefficients
         super().__init__(reactions=reactions, costs=costs)
 
-        if balanced is not None:
-            self.balanced = balanced
-        else:
-            self.balanced = False
+        self.balanced = balanced
 
     def __eq__(self, other):
         if super().__eq__(other):
@@ -57,15 +55,20 @@ class BalancedPathway(BasicPathway):
     ):
         """
         TODO: Implement this method
-        Balances multiple reaction pathways to a net reaction
+
+        Balances multiple reaction pathways to a net reaction.
+
+        NOTE: Currently, to automatically balance and create a BalancedPathway object,
+        you must use the PathwaySolver class.
         """
 
-        pass
-
-    def comp_matrix(self):
+    def comp_matrix(self) -> np.ndarray:
         """
         Internal method for getting the composition matrix used in the balancing
         procedure.
+
+        Returns:
+            An array representing the composition matrix for a reaction
         """
         return np.array(
             [
@@ -77,9 +80,15 @@ class BalancedPathway(BasicPathway):
             ]
         )
 
-    def get_coeff_vector_for_rxn(self, rxn):
+    def get_coeff_vector_for_rxn(self, rxn) -> np.ndarray:
         """
         Internal method for getting the net reaction coefficients vector.
+
+        Args:
+            rxn: Reaction object to get coefficients for
+
+        Returns:
+            An array representing the reaction coefficients vector
         """
         return np.array(
             [
@@ -88,20 +97,27 @@ class BalancedPathway(BasicPathway):
             ]
         )
 
-    def contains_interdependent_rxns(self, precursors):
-        precursors = set(precursors)
+    def contains_interdependent_rxns(self, precursors: List[Composition]) -> bool:
+        """
+        Whether or not the pathway contains interdependent reactions, given a list of
+        provided precursors.
+
+        Args:
+            precursors: List of precursor compositions
+        """
+        precursors_set = set(precursors)
         interdependent = False
 
         rxns = set(self.reactions)
         num_rxns = len(rxns)
 
         if num_rxns == 1:
-            return False, None
+            return False
 
         for combo in limited_powerset(rxns, num_rxns):
             size = len(combo)
             if (
-                any([set(rxn.reactants).issubset(precursors) for rxn in combo])
+                any(set(rxn.reactants).issubset(precursors_set) for rxn in combo)
                 or size == 1
             ):
                 continue
@@ -111,8 +127,8 @@ class BalancedPathway(BasicPathway):
             unique_reactants = []
             unique_products = []
             for rxn in combo:
-                unique_reactants.append(set(rxn.reactants) - precursors)
-                unique_products.append(set(rxn.products) - precursors)
+                unique_reactants.append(set(rxn.reactants) - precursors_set)
+                unique_products.append(set(rxn.products) - precursors_set)
 
             overlap = [False] * size
             for i in range(size):
@@ -129,7 +145,8 @@ class BalancedPathway(BasicPathway):
         return interdependent
 
     @property
-    def average_cost(self):
+    def average_cost(self) -> float:
+        """Returns the mean cost of the pathway"""
         return np.dot(self.coefficients, self.costs) / sum(self.coefficients)
 
     def __repr__(self):
